const request = require('request')
const Promise = require('bluebird')
const sharp = require('sharp')
const {map} = require('async')

function uploadImage(image, visibility = 'PRIVATE', type = 'ALL') {
    if (!process.env.IMAGE_SERVICE_URL) throw new Error('Missing IMAGE_SERVICE_URL env variable')
    const options = {
        url: `${process.env.IMAGE_SERVICE_URL}/v1/generic`,
        headers: {'Content-Type': 'application/json'},
        body: JSON.stringify({
            type: type,
            image: image,
            visibility: visibility,
        })
    }
    return new Promise((resolve, reject) => {
        request.post(options, (err, res, body) => {
            if (err) {
                console.error(err)
                reject(err)
            }
            resolve(JSON.parse(body).links.url)
        })
    })
}

function resizeAndUpload(image, sizes, visibility, type) {

    const encoding = 'base64'
    const resize = ({width, height}, callback) => {
        sharp(new Buffer(image, encoding)).resize(width, height).toBuffer((err, data) => {
            if (err) throw err
            uploadImage(data.toString(encoding), visibility, type).then(url => {
                callback(null, {[`${width}x${height}`]: url})
            }).catch(callback)
        })
    }

    return new Promise((resolve, reject) => {
        const done = (err, res) => {
            if (err) reject(err)
            const result = {links: res.reduce((acc, item) => Object.assign({}, acc, item))}
            resolve(result)
        }
        map(sizes, resize, done)
    })
}

function resizeAndUpload3X(image, sizes, visibility, type) {
    const encoding = 'base64'
    const resize = (size, callback) => {
        Object.keys(size).forEach(sizeKey => {
            const sizeValue = size[sizeKey]
            sharp(new Buffer(image, encoding)).resize(sizeValue, sizeValue).toBuffer((err, data) => {
                if (err) throw err
                uploadImage(data.toString(encoding), visibility, type).then(url => {
                    callback(null, {[sizeKey]: url})
                }).catch(callback)
            })
        })
    }

    return new Promise((resolve, reject) => {
        const done = (err, res) => {
            if (err) reject(err)
            const result = {links: res.reduce((acc, item) => Object.assign({}, acc, item))}
            resolve(result)
        }
        map(sizes, resize, done)
    })
}

exports.uploadImage = uploadImage
exports.resizeAndUpload = resizeAndUpload
exports.resizeAndUpload3X = resizeAndUpload3X